# encoding: UTF-8
#{ COPYRIGHT:
#
#  © 2018 Gordon McIntyre
#
#}

module GM
  module NumberComponent
    # CONSTANTS defined here
    # Module @@variables defined here
    # Classes unique to ThisPlugin defined here

    extend self

    def init()
    # define instance @variables here
    end
    
    def self.findGroups(entities)
      # Looks for groups
      allGroups = []
      entities.each { |entity|
        if entity.is_a?(Sketchup::Group) ||  entity.is_a?(Sketchup::ComponentInstance)
          allGroups << entity
#          allGroups.concat(findGroups(entity.definition.entities) )
        end
      }
      return allGroups
    end #findGroups

    def xy_sort(comps, reverseOrder)
      # Sort the component instances or groups by y-coordinate then x-coordinate
      # Add x and y coords to each group by making an array for each Group
      # that contains [group, x, y] to make sorting easier.
      #
      groupArray = Array.new
      comps.each { |item|
        if item.is_a?(Sketchup::Group) 
          itemBounds = item.local_bounds
        else # ComponentInstance
          itemBounds = item.definition.bounds
        end #if
        centre = itemBounds.center
        max = itemBounds.max
        # Place 0.1 inch above bounds to avoid z-fighting
        centre.z = max.z + 0.1 
        centre.transform! item.transformation
        groupArray << [item, centre]
      }
      # At this point, groupArray has an unsorted array of arrays. 
      # Each groupArray[i] array has a group/component and a point
      # Now sort them by y, then x so numbers read in ascending order
      # from left to right, top to bottom.
      
      if reverseOrder
        smaller = -1
        larger = 1
      else
        smaller = 1
        larger = -1
      end #if
            
#      points.sort_by!{ |p| [p.y, p.x] }
      groupArray.sort! do |a, b|
        case
          when a[1].y < b[1].y
            smaller
          when a[1].y > b[1].y
            larger
          else
            a[1].x <=> b[1].x
        end #case
      end #sort block
      return groupArray
    end #def xy_sort

    def number_components() 
      model = Sketchup.active_model
      model.start_operation "Number Components"
      numberGroup = model.active_entities.add_group
      numberGroup.name = "Component Numbers"
      selection = model.selection
      prompts = ["Text height", "Unit", "Direction"]
      defaults = ["100", "mm", "Top to Bottom"]
      list = ["", "mm|Inches", "Top to Bottom|Bottom to Top"]
      input = UI.inputbox(prompts, defaults, list, "Number Components")
      
      if !input # User cancelled
        return
      end #if
      
      textHeight = input[0].to_f
      
      if input[1] == "mm"
        textHeight /= 25.4
      end #if
      
      if textHeight == 0.0
        textHeight = 100.mm
        UI.messagebox("Invalid size. Defaulting to 100mm")
      end #if

      if input[2] == "Bottom to Top"
        reverseOrder = true
      else
        reverseOrder = false
      end #if
      
      groups = findGroups(selection)
      if groups.size == 0
        UI.messagebox("No groups or components selected.", MB_OK)
        return
      else 
        puts "Found #{groups.size} groups & components."
      end #if
      
      # Now sort the groups by coord
      sortedGroups = xy_sort(groups, reverseOrder)

      count = 1
      sortedGroups.each { |item|
        tString = count.to_s
        tGroup = numberGroup.entities.add_group
        tGroup.name = tString
        
      
        tGroup.entities.add_3d_text(tString, TextAlignCenter, "Arial",
          false, false, textHeight, 0.0, 0, true, 0.0)
        # Move text so it is centred on the origin
        tCentre = tGroup.local_bounds.center
        tCentre.z = 0 # ignore height
        textTransform = Geom::Transformation.new(tCentre).inverse
        tGroup.transform! textTransform
        
        # Now move text to centre it on the current item's centre
        tGroup.transform! item[1] # item[1] is a point at centre of group/component
        count += 1
      }
      model.commit_operation
    end #def number_components()
    
    def validateObject
      # The number of groups and components selected could be
      # counted here, but it was decided not to do so in 
      # case it slowed this method down.
      selection = Sketchup.active_model.selection
        if selection.size > 0
            return true
        else
            return false
        end
    end #validateObject
   
    # Run Once at startup block here:
    this_file = Module::nesting[0].name << ':'<< File.basename(__FILE__)
    unless file_loaded?( this_file )
    init()
    UI.add_context_menu_handler do |menu|
      menu.add_separator if validateObject
      menu.add_item("Number Components") {self.number_components} if validateObject
    end #do

    file_loaded( this_file )
    end #unless file_loaded
    
  end # NumberComponent
end #GM
